﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

/*
    valid ma types: sma, ema, dema, tema, trima, t3, mama, kama, wma

    valid signal types:
     - rsi-level (rsi goes overbought/-sold)
     - ma-level (ma goes overbought/-sold)
     - rsi-level-cross (rsi goes ob/os and comes back)
     - ma-level-cross (ma goes ob/os and comes back)
     - ma-rsi-cross (ma crosses rsi)
     - rsi-50 (rsi crosses 50)
     - ma-50 (ma crosses 50
*/

namespace PshaisIndicators
{
    public class pshaisRSI : IIndicatorScript
    {
        int rsi_length = 9;
        int ma_length = 5;
        Core.MAType ma_type = Core.MAType.Sma;
        double cross_threshold = 0.0;

        double rsi_overbought = 80;
        double rsi_oversold = 20;

        const int SIGNAL_TYPE1 = 0; // rsi goes overbought/-sold
        const int SIGNAL_TYPE2 = 1; // ma goes overbought/-sold
        const int SIGNAL_TYPE3 = 2; // rsi goes ob/os and comes back
        const int SIGNAL_TYPE4 = 3; // ma goes ob/os and comes back
        const int SIGNAL_TYPE5 = 4; // ma crosses rsi
        const int SIGNAL_TYPE6 = 5; // rsi crosses 50
        const int SIGNAL_TYPE7 = 6; // ma crosses 50

        int signal_type = SIGNAL_TYPE1;

        double rsi_value, ma_value;

        // Properties
        public string Name
        {
            get { return "pshai's RSI"; }
        }
        public int TicksBack
        {
            get { return Core.RsiLookback(rsi_length) + Core.MovingAverageLookback(ma_length, ma_type) + 75; }
        }

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> inputField = new List<ScriptParameter>
            {
                new ScriptParameter("RSI Length", ScriptParameterType.Integer, rsi_length.ToString()),
                new ScriptParameter("RSI Overbought", ScriptParameterType.Decimal, rsi_overbought.ToString()),
                new ScriptParameter("RSI Oversold", ScriptParameterType.Decimal, rsi_oversold.ToString()),
                new ScriptParameter("MA Length", ScriptParameterType.Integer, ma_length.ToString()),
                new ScriptParameter("MA Type", ScriptParameterType.String, ma_type.ToString()),
                new ScriptParameter("MA Cross Thres. %", ScriptParameterType.Decimal, cross_threshold.ToString()),
                new ScriptParameter("Signal Type (value 0-6)", ScriptParameterType.Integer, signal_type.ToString()),
            };

            return inputField;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            rsi_length = ParseInt(parameters["RSI Length"], rsi_length);
            rsi_overbought = ParseDouble(parameters["RSI Overbought"], rsi_overbought);
            rsi_oversold = ParseDouble(parameters["RSI Oversold"], rsi_oversold);

            ma_length = ParseInt(parameters["MA Length"], ma_length);
            ma_type = ParseMAType(parameters["MA Type"]);
            cross_threshold = ParseDouble(parameters["MA Cross Thres. %"], cross_threshold);

            signal_type = ParseInt(parameters["Signal Type (value 0-6)"], signal_type);
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 1, Name = "Overbought", HexLineColor = "#882222" },
                new ChartRecord() { ChartIndex = 1, Name = "Oversold", HexLineColor = "#228822" },
                new ChartRecord() { ChartIndex = 1, Name = "Mid50", HexLineColor = "#AAAAAA" },
                new ChartRecord() { ChartIndex = 1, Name = string.Format("RSI({0})", rsi_length), HexLineColor = "#FF9922" },
                new ChartRecord() { ChartIndex = 1, Name = string.Format("{0}({1})", ma_type, ma_length), HexLineColor = "#FF4444" },
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                Convert.ToDecimal(rsi_overbought),
                Convert.ToDecimal(rsi_oversold),
                50m,
                Convert.ToDecimal(rsi_value),
                Convert.ToDecimal(ma_value),
            };

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
            // In this code example we do not have to do anything
        }

        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                int lb1 = Core.RsiLookback(rsi_length);
                int lb2 = Core.MovingAverageLookback(ma_length, ma_type);
                int position = instrument.Close.Count() - (1 + context.Offset);
                var result = IndicatorResult.StayNoPosition;
                var mult = 100000000;
                var close = instrument.Close.Select(x => x * mult).ToArray();

                int rsi_b, rsi_l;
                var rsi_o = new double[lb1 + 51];
                var rsi_r = Core.Rsi(position - (lb1 + 50), position, close, rsi_length, out rsi_b, out rsi_l, rsi_o);

                int ma_b, ma_l;
                var ma_o = new double[rsi_o.Length];
                var ma_r = Core.MovingAverage(1, rsi_o.Length - 1, rsi_o, ma_length, ma_type, out ma_b, out ma_l, ma_o);

                if (rsi_r == Core.RetCode.Success && ma_r == Core.RetCode.Success)
                {
                    rsi_value = rsi_o[rsi_l - 1];
                    ma_value = ma_o[ma_l - 1];
                    
                    switch (signal_type)
                    {
                        // rsi goes overbought/-sold
                        case SIGNAL_TYPE1:
                            {
                                if (rsi_value >= rsi_overbought)
                                    result = IndicatorResult.SellShort;

                                else if (rsi_value <= rsi_oversold)
                                    result = IndicatorResult.BuyLong;
                            }
                            break;

                        // ma goes overbought/-sold
                        case SIGNAL_TYPE2:
                            {
                                if (ma_value >= rsi_overbought)
                                    result = IndicatorResult.SellShort;

                                else if (ma_value <= rsi_oversold)
                                    result = IndicatorResult.BuyLong;
                            }
                            break;

                        // rsi goes ob/os and comes back
                        case SIGNAL_TYPE3:
                            {
                                var rsi_prev = rsi_o[rsi_l - 2];

                                if (rsi_prev > rsi_overbought && rsi_value < rsi_overbought)
                                    result = IndicatorResult.SellShort;

                                else if (rsi_prev < rsi_oversold && rsi_value > rsi_oversold)
                                    result = IndicatorResult.BuyLong;
                            }
                            break;

                        // ma goes ob/os and comes back
                        case SIGNAL_TYPE4:
                            {
                                var ma_prev = ma_o[ma_l - 2];

                                if (ma_prev > rsi_overbought && ma_value < rsi_overbought)
                                    result = IndicatorResult.SellShort;

                                else if (ma_prev < rsi_oversold && ma_value > rsi_oversold)
                                    result = IndicatorResult.BuyLong;
                            }
                            break;

                        // ma crosses rsi
                        case SIGNAL_TYPE5:
                            {
                                if (rsi_value * (1 - cross_threshold / 100) > ma_value)
                                    result = IndicatorResult.BuyLong;

                                else if (rsi_value * (1 + cross_threshold / 100) < ma_value)
                                    result = IndicatorResult.SellShort;
                            }
                            break;

                        // rsi crosses 50
                        case SIGNAL_TYPE6:
                            {
                                if (rsi_value > 50)
                                    result = IndicatorResult.BuyLong;

                                else if (rsi_value < 50)
                                    result = IndicatorResult.SellShort;
                            }
                            break;

                        // ma crosses 50
                        case SIGNAL_TYPE7:
                            {
                                if (ma_value > 50)
                                    result = IndicatorResult.BuyLong;

                                else if (ma_value < 50)
                                    result = IndicatorResult.SellShort;
                            }
                            break;

                        default:
                            result = IndicatorResult.StayNoPosition;
                            break;
                    }

                    return result;
                }
                else
                {
                    context.Logger.LogToFile(string.Format("TA failed: {0} : {1}", rsi_r, ma_r));
                }
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }

        public static Core.MAType ParseMAType(object value)
        {
            Core.MAType val;
            switch (value.ToString().ToLower())
            {
                case "sma":
                    val = Core.MAType.Sma;
                    break;
                case "ema":
                    val = Core.MAType.Ema;
                    break;
                case "dema":
                    val = Core.MAType.Dema;
                    break;
                case "tema":
                    val = Core.MAType.Tema;
                    break;
                case "trima":
                    val = Core.MAType.Trima;
                    break;
                case "t3":
                    val = Core.MAType.T3;
                    break;
                case "mama":
                    val = Core.MAType.Mama;
                    break;
                case "kama":
                    val = Core.MAType.Kama;
                    break;
                case "wma":
                    val = Core.MAType.Wma;
                    break;
                default:
                    val = Core.MAType.Sma;
                    break;
            }

            return val;
        }

        int ParseInt(object value, int lastValue)
        {
            try
            {
                return Convert.ToInt32(value.ToString());
            }
            catch
            {
                return lastValue;
            }
        }

        double ParseDouble(object value, double lastValue)
        {
            var numStyle = System.Globalization.NumberStyles.Any;
            var cult = System.Globalization.CultureInfo.InvariantCulture;

            try
            {
                double output;
                double.TryParse(
                    value.ToString().Replace(',', '.'),
                    numStyle,
                    cult,
                    out output);
                return output;
            }
            catch
            {
                return lastValue;
            }
        }

        string ParseString(object value, string lastValue)
        {
            try
            {
                return Convert.ToString(value);
            }
            catch
            {
                return lastValue;
            }
        }
    }
}
